@@ -1,8 +1,8 @@
 BiCopPar2Tau<-function(family,par,par2=0)
 {
-	if(!(family %in% c(0,1,2,3,4,5,6,7,8,9,10,13,14,16,17,18,19,20,23,24,26,27,28,29,30,33,34,36,37,38,39,40))) stop("Copula family not implemented.")
+	if(!(family %in% c(0,1,2,3,4,5,6,7,8,9,10,13,14,16,17,18,19,20,23,24,26,27,28,29,30,33,34,36,37,38,39,40, 41,42,51,61,71))) stop("Copula family not implemented.")
 	if(c(7,8,9,10,17,18,19,20,27,28,29,30,37,38,39,40) %in% family && par2==0) stop("For t-, BB1, BB6, BB7 and BB8 copulas, 'par2' must be set.")
-	if(c(1,3,4,5,6,13,14,16,23,24,26,33,34,36) %in% family && length(par)<1) stop("'par' not set.")
+	if(c(1,3,4,5,6,13,14,16,23,24,26,33,34,36,41,51,61,71) %in% family && length(par)<1) stop("'par' not set.")
 	
 	if((family==1 || family==2) && abs(par[1])>=1) stop("The parameter of the Gaussian and t-copula has to be in the interval (-1,1).")
 	#if(family==2 && par2<=2) stop("The degrees of freedom parameter of the t-copula has to be larger than 2.")
@@ -29,6 +29,16 @@ BiCopPar2Tau<-function(family,par,par2=0)
 	if((family==29 || family==39) && par2>=0) stop("The second parameter of the rotated BB7 copula has to be negative.")
 	if((family==30 || family==40) && par>-1) stop("The first parameter of the rotated BB8 copula has to be in the interval (-oo,-1].")
 	if((family==30 || family==40) && (par2>=0 || par2<(-1))) stop("The second parameter of the rotated BB8 copula has to be in the interval [-1,0).")
+	if((family==41 || family==51) && par<=0) stop("The parameter of the reflection asymmetric copula has to be positive.")
+	if((family==61 || family==71) && par>=0) stop("The parameter of the rotated reflection asymmetric copula has to be negative.")
+	if(family==42)
+	{
+		a=par
+		b=par2
+		limA=(b - 3 - sqrt(9 + 6 * b - 3 * b^2))/2
+		if(abs(b)>1) stop("The second parameter of the two-parametric asymmetric copulas has to be in the interval [-1,1]")
+		if(a>1 || a<limA) stop("The first parameter of the two-parametric asymmetric copula has to be in the interval [limA(par2),1]")
+	}
   
   if(family==0)
 	{
@@ -60,7 +70,11 @@ BiCopPar2Tau<-function(family,par,par2=0)
 	}
 	else if(family==6 || family==16)
 	{
-		tau=1+4/par^2*integrate(function(x) log(x)*x*(1-x)^(2*(1-par)/par), 0, 1)$value
+		#tau=1+4/par^2*integrate(function(x) log(x)*x*(1-x)^(2*(1-par)/par), 0, 1)$value
+		param1=2/par+1
+		tem=digamma(2)-digamma(param1)
+		tau=1+tem*2/(2-par)
+		tau[par==2]=1-trigamma(2)
 	}
 	else if(family==7 || family==17)
 	{
@@ -105,7 +119,13 @@ BiCopPar2Tau<-function(family,par,par2=0)
 	}
 	else if(family==26 || family==36)
 	{
-    tau=-1-4/par^2*integrate(function(x) log(x)*x*(1-x)^(2*(1+par)/-par), 0, 1)$value
+		#tau=-1-4/par^2*integrate(function(x) log(x)*x*(1-x)^(2*(1+par)/-par), 0, 1)$value
+		theta=-par
+		param1=2/theta+1
+		tem=digamma(2)-digamma(param1)
+		tau=1+tem*2/(2-theta)
+		tau[theta==2]=1-trigamma(2)
+		tau=-tau
 	}
 	else if(family==27 || family==37)
 	{
@@ -139,6 +159,25 @@ BiCopPar2Tau<-function(family,par,par2=0)
 		tau=1+4*integrate(kt,0,1)$value
 		tau=-tau
 	}
+	else if(family==41 || family==51)
+	{
+		de=par
+		ln2=log(2)
+		tem=(2-2*de)*ln2+lgamma(2*de)-2*lgamma(1+de)
+		tau=1-de*exp(tem)
+	}
+	else if(family==61 || family==71)
+	{
+		de=-par
+		ln2=log(2)
+		tem=(2-2*de)*ln2+lgamma(2*de)-2*lgamma(1+de)
+		tau=1-de*exp(tem)
+		tau=-tau
+	}
+	else if(family==42)
+	{
+		tau=(75*par2-par2^2+par*(25-par2))/450
+	}
 
 return(tau)
 }
\ No newline at end of file
